import {GameModel} from "../GameModel";
import {ItemGroup} from "../inventory/ItemGroup";
import {AbstractItemGroup} from "../inventory/AbstractItemGroup";
import {ConcreteItemGroup} from "../inventory/ConcreteItemGroup";

export class ItemgroupUtilities {

  public static findItemgroupPathToItem(searchedItemName: string, itemgroups: ItemGroup[]): ItemGroup[] {
    for(let i=0; i<itemgroups.length; i++) {
      const found_path = this.findItemgroupPathToItemRecursive(itemgroups[i], searchedItemName, []);
      if(found_path.length > 0) {
        return found_path;
      }
    }
    return [];
  }

  public static findItemgroupPathToItemgroup(searchedItemgroupName: string, itemgroups: ItemGroup[]): ItemGroup[] {
    for(let i=0; i<itemgroups.length; i++) {
      const found_path = this.findItemgroupPathToItemgroupRecursive(itemgroups[i], searchedItemgroupName, []);
      if(found_path.length > 0) {
        return found_path;
      }
    }
    return [];
  }

  private static findItemgroupPathToItemgroupRecursive(root: ItemGroup, searchedItemgroupName: string, path: ItemGroup[]): ItemGroup[] {
    path.push(root);

    if(root.componentName === searchedItemgroupName) {
      return path;
    }

    if(root instanceof AbstractItemGroup) {
      for(let i=0; i<root.children.length; i++) {
        const found_path = this.findItemgroupPathToItemgroupRecursive(root.children[i], searchedItemgroupName, path.concat())
        if(found_path != undefined && found_path.length > 0) {
          return found_path;
        }
      }
      return [];
    } else {
      return [];
    }
  }

  private static findItemgroupPathToItemRecursive(root: ItemGroup, searchedItemName: string, path: ItemGroup[]): ItemGroup[] {
    path.push(root);

    if(root instanceof ConcreteItemGroup) {
      const searchedItem = root.items.find(item => item.componentName === searchedItemName)
      if(searchedItem != undefined) {
        return path;
      }
    } else if(root instanceof AbstractItemGroup) {
      for(let i=0; i<root.children.length; i++) {
        const found_path = this.findItemgroupPathToItemRecursive(root.children[i], searchedItemName, path.concat());
        if(found_path.length > 0) {
          return found_path;
        }
      }
    }
    return [];
  }

}
